﻿local mod = ExtraCD
local L = LibStub("AceLocale-3.0"):GetLocale("ExtraCD")
local LSM = LibStub("LibSharedMedia-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local ecddb
local CLASS

local function setOption(info, value)
	local name = info[#info]
	ecddb[name] = value
	mod:ResetAllIcons()
end

local function getOption(info)
	local name = info[#info]
	return ecddb[name]
end

local function newSpellTable(data)
	local t = {}
	if type(data) == "table" then 
		for k, v in pairs(data) do
			t[k] = v
		end
	else
		t.type = "custom"
	end
	t.enable = true
	return t
end

function mod:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata("ExtraCD", "Title"))
end

function mod:AddOption(name, pName, keyName)
	return AceConfigDialog:AddToBlizOptions("ExtraCD", name, pName, keyName)
end

function mod:ChangeProfile()
	ecddb = self.db1.profile
end

function mod:AddDataOption(spellId)
	local name, _, icon = GetSpellInfo(spellId)
	local dbKey = tostring(spellId)
	local db = ecddb.spells
	if not name then 
		mod:log(L["spell not exists, id:"] .. spellId)
		db[dbKey] = nil
		return 
	end	
	local isOriginal = ( db[dbKey].type ~= "custom" ) and true or false
	local op = self.options.args.advance.args.data.args
	op[dbKey] = {
		type = "group",
		name = function() local s = name if db[dbKey].type == "custom" then s = " * " .. name end if not db[dbKey].enable then s = "|cffDD1133" .. s .. "|r" end return s end,
		icon = icon,
		args = {
			enable = {
				type = "toggle",
				desc = function() GameTooltip:SetHyperlink(GetSpellLink(spellId)) end,
				descStyle = "custom",
				name = function() if not db[dbKey].desc or db[dbKey].desc == "" then return "\124T"..icon..":24\124t" .. name else return  "\124T"..icon..":24\124t" .. name ..  " (" .. (db[dbKey].desc).. ")"  end end,
				set = function(info, value) db[dbKey].enable = value self:ResetAllIcons() end,
				get = function() return db[dbKey].enable end,
				width = "full",
				order = 1,
			},
			header = {
				type = "header",
				name = "",
				order = 2,
			},
			--[[class = {					
				name = L["Class"],
				type = "select",
				values = mod.CLASS,
				disabled = function() return ecddb.dataLock end,
				get = function() return db[dbKey].class end,
				set = function(info, value) db[dbKey].class = value end,
			},]]
			--[[type = {
				name = L["Type"],
				type = "select",
				values = mod.CD_TYPE,
				disabled = function() return ecddb.dataLock end,
				get = function() return db[dbKey].type end,
				set = function(info, value) db[dbKey].type = value end,
			},]]
			cd = {
				name = L["CD"],
				type = "input",
				disabled = function() return ecddb.dataLock and isOriginal end,
				get = function() return tostring(db[dbKey].cd or 0) end,
				set = function(info, value) db[dbKey].cd = tonumber(value) self:ResetAllIcons() end,
				pattern = "%d+$",
			},
			duration = {
				name = L["Duration"],
				type = "input",
				disabled = function() return ecddb.dataLock and isOriginal end,
				get = function() return tostring(db[dbKey].duration or 0) end,
				set = function(info, value) db[dbKey].duration = tonumber(value) self:ResetAllIcons() end,
				pattern = "%d+$",
			},
			spellId = {
				name = L["Spell ID"],
				type = "input",
				disabled = true,
				get = function() return dbKey end,
				pattern = "%d+$",
			},
			delete = {
				name = L["DELETE"],
				type = "execute",
				disabled = isOriginal,
				confirm = true,
				confirmText = L["Are you sure to delete the data?"],
				func = function() db[dbKey] = nil op[dbKey] = nil self:ResetAllIcons() end,
				order = -1,
			},
		},
	}
end

function mod:OnOptionCreate()
	
	ecddb = self.db1.profile
	local newSpellId, newDataName
	self.options = {
		type = "group",
		name = GetAddOnMetadata("ExtraCD", "Title"),
		args = {
			general = {
				type = "group",
				name = "General",
				get = getOption,
				set = setOption,
				order = 1,
				args = {
					ecddesc1 = {
						type = "description",
						order = 0,
						name = GetAddOnMetadata("ExtraCD", "Notes"),
					},
					ecdheader1 = {
						type = "header",
						order = 1,
						name = "",
					},
					autoload = {
						type = "group",
						order = 2,
						inline = true,
						name = L["Automatically scan and load icds of the types below"],
						args = {
							talent = {
								name = L["talent"],
								desc = L["Scan and show internal cooldown of player's talent."],
								type = "toggle",
								order = 100,
							},
							enchant = {
								name = L["enchant"],
								desc = L["Scan and show internal cooldown of player's enchant."],
								type = "toggle",
								order = 200,
							},
							item = {
								name = L["item"],
								desc = L["Scan and show internal cooldown of player's trinket and special weapons."],
								type = "toggle",
								order = 300,
							},
							itemset = {
								name = L["item set"],
								desc = L["Scan and show internal cooldown of player's item set."],
								type = "toggle",
								order = 301,
							},
							spec = {
								name = L["spec"],
								desc = L["Scan and show internal cooldown of player's specializations."],
								type = "toggle",
								order = 302,
							}
						},
					},
					
					ecdheader2 = {
						type = "header",
						order = 304,
						name = ""
					},
					lock = {
						name = L["Lock frame"],
						desc = L["Set the frame locked."],
						type = "toggle",
						order = 305,
						set = function(info, value) self.bar:SetMovable(not value) ecddb.lock = value end,
					},
					combat = {
						name = L["Combat only"],
						desc = L["Show the icon only in combat."],
						type = "toggle",
						order = 306,
					},
					tip = {
						name = L["Show tooltip"],
						desc = L["Show tooltip for the icons."],
						type = "toggle",
						order = 310,
					},
					rowmax = {
						name = L["Icons each row"],
						desc = L["Set the icons in each row"],
						type = "range",
						max = 20,
						min = 1,
						step = 1,
						order = 333,
					},
					textOpacity = {
						name = L["Text opacity"],
						type = "range",
						order = 312,
						max = 1,
						min = 0,
						step = 0.1,
					},
					glowOpacity = {
						type = "range",
						order = 315,
						name = L["Glow opacity"],
						max = 1,
						min = 0,
						step = 0.1,
					},
					textsize = {
						name = L["Text size"],
						desc = L["Set the cooldown text size"],
						type = "range",
						max = 30,
						min = 12,
						step = 1,
						order = 320,				
					},
					textfont = {
						name = L["Text font"],
						desc = L["Set the font of the text"],
						type = "select",
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						order = 321,
					},
					iconinterval = {
						name = L["Icon interval"],
						desc = L["Set the interval size between icons"],
						type ="range",
						max = 20,
						min = 0,
						step = 1,
						order = 323,
					},
					iconsize = {
						name = L["Icon size"],
						desc = L["Set the icon size"],
						type = "range",
						max = 60,
						min = 1,
						step = 1,
						order = 330,					
					},
					ecdheader4 = {
						type = "header",
						name = "",
						order = 348,
					},
					ecddesc2 = {
						type = "description",
						name = L["When you crtl+right click at the icon, it will be disabled. If you want to show it again, enable the spell in the list in advance option. The disabled spells will be at the end of the spell list."],
						order = 349,
					},
					iconborder = {
						name = L["Icon border"],
						desc = L["Set the icon border"],
						type = "select",
						dialogControl = 'LSM30_Border',
						values = LSM:HashTable("border"),
						order = 340,
					},
				},
			},	
			advance = {
				type = "group",
				name = "Advance",
				childGroups = "tab",
				args = {
					data = {
						type = "group",
						name = "Data",
						args = {
							ecddesc4 = {
								type = "description",
								name = L["Data options provide more customized options for adding new cds and modifying the original cds"],
								order = 1,
							},
							ecddesc3 = {
								type = "description",
								name = L["Add a new custom cd(no limit of being an icd), and manage it by yourself.(marked with '*')"],
								order = 5
							},
							ecdheader5 = {
								type = "header",
								name = "",
								order = 2,
							},
							--[[name = {
								name = L["Name"],
								type = "input",
								set = function(info, value) newDataName = value end,
								get = function() return newDataName end,
								pattern = "%w+$",
								order = 9,
							},]]
							spellId = {
								name = L["Spell ID"],
								type = "input",
								set = function(info, value) newSpellId = value end,
								get = function() return newSpellId end,
								pattern = "%d+$",
								order = 8,
							},
							addCustom = {
								type = "execute",
								name = L["Add New CDs"],
								func = function()
									if not newSpellId then return end
									if not ecddb.spells[newSpellId] then
										ecddb.spells[newSpellId] = newSpellTable()
										self:AddDataOption(tonumber(newSpellId))
									else
										self:log("Name '" .. newSpellId .. "' " .. L["already exists."])
									end
								end,
								order = 10,
							},
							ecdheader6 = {
								type = "header",
								name = "",
								order = 40,
							},
							dataReset = {
								type = "execute",
								name = "Reset Original Data",
								desc = L["To reset all addon original data, your custom data and proc test data will not be changed"],
								confirm = true,
								confirmText = L["Are you sure to reset all addon original data?"],
								order = 100,
								func = function()
									local data = self:GetSpellData()
									local db = ecddb.spells
									local op = self.options.args.advance.args.data.args
									for k, v in pairs(db) do
										if v.type ~= "custom" then
											db[k] = nil
											op[k] = nil
										end
									end
									for k, v in pairs(data) do
										k = tostring(k)
										db[k] = newSpellTable(v)
									end
									for k, v in pairs(db) do
										self:AddDataOption(tonumber(k))
									end
									self:ResetAllIcons()
								end,
							},
							dataDelete = {
								type = "execute",
								name = "Delete Custom Data",
								desc = L["To delete all custom data, the addon original data and proc test data will not be changed"],
								confirm = true,
								confirmText = L["Are you sure to delete all custom data?"],
								order = 120,
								func = function()
									local op = self.options.args.advance.args.data.args
									local data = self:GetSpellData()
									local db = ecddb.spells
									for k, v in pairs(db) do
										if v.type == "custom" then
											db[k] = nil
											op[k] = nil
										end
									end
									self:ResetAllIcons()
								end,
							},
							dataLock = {
								type = "execute",
								set = setOption,
								get = getOption,
								desc = L["To unlock or lock the original data. It's suggested that you should not modify the addon original data, some advance users may need this feature."],
								name = function() if ecddb.dataLock then return L["Unlock Original Data"] else return L["Lock Original Data"] end end,
								func = function() ecddb.dataLock = not ecddb.dataLock end,
								order = 140,
							},
							ecdheader9 = {
								type = "header",
								order = 380,
								name = "",
							},
						}
					},
					ProcTest = {
						type = "group",
						name = L["Proc Test"],
						get = getOption,
						set = setOption,
						disabled = true,
						args = {
							ecddesc8 = {
								type = "description",
								name = L["Proc test provides the feature of testing coverage of proc and estimating the real icds of the unknown"],
								order = 1,
							},
							ecdheader9 = {
								type = "header",
								order = 2,
								name = "",
							},
							ecddesc = {
								type = "description",
								name = L["Add a new proc test(no limit of being a proc), and manage it by yourself."],
								order = 5
							},
							addProc = {
								type = "execute",
								name = L["Add New Proc Test"],
								order = 10,
							},
						}
					}
				}
			},
			
		}
	}
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db1)
	self.options.args.profiles.order = -1
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ExtraCD", ExtraCD.options)
	self:AddOption(GetAddOnMetadata("ExtraCD", "Title"), nil, "general")
	
	-- add addon data
	do
		local data = self:GetSpellData()
		local db = ecddb.spells
				
		-- initialize db
		local dVersion = self:GetDataVersion()
		if ecddb.dataVersion < dVersion then
			for k, v in pairs(db) do
				if v.type ~= "custom" then
					db[k] = nil
				end
			end
			ecddb.dataVersion = dVersion
			for k, v in pairs(data) do
				k = tostring(k)
				db[k] = newSpellTable(v)
			end
		end
		
		-- add options
		for k, v in pairs(db) do
			self:AddDataOption(tonumber(k))
		end
	end
	
	self:AddOption(L["Advance"], "ExtraCD", "advance")
	
	self:AddOption(L["Profiles"], "ExtraCD", "profiles")
end